@@ -3,6 +3,8 @@
 # and distributed under the terms of the GNU General Public Licence,
 # either version 2, or version 3 (at your option).
 
+from __future__ import print_function
+
 """Read osc configuration and store it in a dictionary
 
 This module reads and parses ~/.oscrc. The resulting configuration is stored
@@ -35,19 +37,30 @@ The configuration dictionary could look like this:
 """
 
 import base64
-import cookielib
-import httplib
 import os
 import re
 import sys
-import StringIO
-import urllib
-import urllib2
-import urlparse
 
-import OscConfigParser
+try:
+    from http.cookiejar import LWPCookieJar, CookieJar
+    from http.client import HTTPConnection, HTTPResponse
+    from io import StringIO
+    from urllib.parse import urlsplit
+    from urllib.error import URLError
+    from urllib.request import HTTPBasicAuthHandler, HTTPCookieProcessor, HTTPPasswordMgrWithDefaultRealm, ProxyHandler
+    from urllib.request import AbstractHTTPHandler, build_opener, proxy_bypass
+except ImportError:
+    #python 2.x
+    from cookielib import LWPCookieJar, CookieJar
+    from httplib import HTTPConnection, HTTPResponse
+    from StringIO import StringIO
+    from urlparse import urlsplit
+    from urllib2 import URLError, HTTPBasicAuthHandler, HTTPCookieProcessor, HTTPPasswordMgrWithDefaultRealm, ProxyHandler
+    from urllib2 import AbstractHTTPHandler, build_opener, proxy_bypass
+
+from . import OscConfigParser
 from osc import oscerr
-from oscsslexcp import NoSecureSSLError
+from .oscsslexcp import NoSecureSSLError
 
 GENERIC_KEYRING = False
 GNOME_KEYRING = False
@@ -77,7 +90,7 @@ def _get_processors():
     """
     try:
         return os.sysconf('SC_NPROCESSORS_ONLN')
-    except ValueError, e:
+    except ValueError as e:
         return 1
 
 DEFAULTS = {'apiurl': 'https://api.opensuse.org',
@@ -90,7 +103,7 @@ DEFAULTS = {'apiurl': 'https://api.opensuse.org',
             # build type settings
             'build-cmd': '/usr/bin/build',
             'build-type': '',                   # may be empty for chroot, kvm or xen
-            'build-root': '/var/tmp/build-root',
+            'build-root': '/var/tmp/build-root/%(repo)s-%(arch)s',
             'build-uid': '',                    # use the default provided by build
             'build-device': '',                 # required for VM builds
             'build-memory': '',                 # required for VM builds
@@ -103,6 +116,8 @@ DEFAULTS = {'apiurl': 'https://api.opensuse.org',
             'builtin_signature_check': '1',     # by default use builtin check for verify pkgs
             'icecream': '0',
 
+            'buildlog_strip_time': '0',  # strips the build time from the build log
+
             'debug': '0',
             'http_debug': '0',
             'http_full_debug': '0',
@@ -110,8 +125,8 @@ DEFAULTS = {'apiurl': 'https://api.opensuse.org',
             'verbose': '1',
             'traceback': '0',
             'post_mortem': '0',
-            'use_keyring': '1',
-            'gnome_keyring': '1',
+            'use_keyring': '0',
+            'gnome_keyring': '0',
             'cookiejar': '~/.osc_cookiejar',
             # fallback for osc build option --no-verify
             'no_verify': '0',
@@ -165,7 +180,7 @@ config = DEFAULTS.copy()
 boolean_opts = ['debug', 'do_package_tracking', 'http_debug', 'post_mortem', 'traceback', 'check_filelist', 'plaintext_passwd',
     'checkout_no_colon', 'checkout_rooted', 'check_for_request_on_action', 'linkcontrol', 'show_download_progress', 'request_show_interactive',
     'review_inherit_group', 'use_keyring', 'gnome_keyring', 'no_verify', 'builtin_signature_check', 'http_full_debug',
-    'include_request_from_project', 'local_service_run']
+    'include_request_from_project', 'local_service_run', 'buildlog_strip_time']
 
 api_host_options = ['user', 'pass', 'passx', 'aliases', 'http_headers', 'email', 'sslcertck', 'cafile', 'capath', 'trusted_prj']
 
@@ -232,6 +247,9 @@ apiurl = %(apiurl)s
 # This should not be 0
 # build-uid =
 
+# strip leading build time information from the build log
+# buildlog_strip_time = 1
+
 # extra packages to install when building packages locally (osc build)
 # this corresponds to osc build's -x option and can be overridden with that
 # -x '' can also be given on the command line to override this setting, or
@@ -353,13 +371,13 @@ cookiejar = None
 
 def parse_apisrv_url(scheme, apisrv):
     if apisrv.startswith('http://') or apisrv.startswith('https://'):
-        return urlparse.urlsplit(apisrv)[0:2]
+        return urlsplit(apisrv)[0:2]
     elif scheme != None:
         # the split/join is needed to get a proper url (e.g. without a trailing slash)
-        return urlparse.urlsplit(urljoin(scheme, apisrv))[0:2]
+        return urlsplit(urljoin(scheme, apisrv))[0:2]
     else:
         msg = 'invalid apiurl \'%s\' (specify the protocol (http:// or https://))' % apisrv
-        raise urllib2.URLError(msg)
+        raise URLError(msg)
 
 
 def urljoin(scheme, apisrv):
@@ -401,8 +419,8 @@ def get_apiurl_usr(apiurl):
     try:
         return get_apiurl_api_host_options(apiurl)['user']
     except KeyError:
-        print >>sys.stderr, 'no specific section found in config file for host of [\'%s\'] - using default user: \'%s\'' \
-            % (apiurl, config['user'])
+        print('no specific section found in config file for host of [\'%s\'] - using default user: \'%s\'' \
+            % (apiurl, config['user']), file=sys.stderr)
         return config['user']
 
 
@@ -422,23 +440,23 @@ def _build_opener(url):
         return _build_opener.last_opener[1]
 
     # respect no_proxy env variable
-    if urllib.proxy_bypass(apiurl):
+    if proxy_bypass(apiurl):
         # initialize with empty dict
-        proxyhandler = urllib2.ProxyHandler({})
+        proxyhandler = ProxyHandler({})
     else:
         # read proxies from env
-        proxyhandler = urllib2.ProxyHandler()
+        proxyhandler = ProxyHandler()
 
     # workaround for http://bugs.python.org/issue9639
-    authhandler_class = urllib2.HTTPBasicAuthHandler
+    authhandler_class = HTTPBasicAuthHandler
     if sys.version_info >= (2, 6, 6) and sys.version_info < (2, 7, 1) \
-        and not 'reset_retry_count' in dir(urllib2.HTTPBasicAuthHandler):
-        print >>sys.stderr, 'warning: your urllib2 version seems to be broken. ' \
-            'Using a workaround for http://bugs.python.org/issue9639'
+        and not 'reset_retry_count' in dir(HTTPBasicAuthHandler):
+        print('warning: your urllib2 version seems to be broken. ' \
+            'Using a workaround for http://bugs.python.org/issue9639', file=sys.stderr)
 
-        class OscHTTPBasicAuthHandler(urllib2.HTTPBasicAuthHandler):
+        class OscHTTPBasicAuthHandler(HTTPBasicAuthHandler):
             def http_error_401(self, *args):
-                response = urllib2.HTTPBasicAuthHandler.http_error_401(self, *args)
+                response = HTTPBasicAuthHandler.http_error_401(self, *args)
                 self.retried = 0
                 return response
 
@@ -447,8 +465,8 @@ def _build_opener(url):
                 return None
 
         authhandler_class = OscHTTPBasicAuthHandler
-    elif sys.version_info >= (2, 6, 6) and sys.version_info < (2, 7, 1):
-        class OscHTTPBasicAuthHandler(urllib2.HTTPBasicAuthHandler):
+    elif sys.version_info >= (2, 6, 6) and sys.version_info < (2, 7, 99):
+        class OscHTTPBasicAuthHandler(HTTPBasicAuthHandler):
             def http_error_404(self, *args):
                 self.reset_retry_count()
                 return None
@@ -457,12 +475,12 @@ def _build_opener(url):
     elif sys.version_info >= (2, 6, 5) and sys.version_info < (2, 6, 6):
         # workaround for broken urllib2 in python 2.6.5: wrong credentials
         # lead to an infinite recursion
-        class OscHTTPBasicAuthHandler(urllib2.HTTPBasicAuthHandler):
+        class OscHTTPBasicAuthHandler(HTTPBasicAuthHandler):
             def retry_http_basic_auth(self, host, req, realm):
                 # don't retry if auth failed
                 if req.get_header(self.auth_header, None) is not None:
                     return None
-                return urllib2.HTTPBasicAuthHandler.retry_http_basic_auth(self, host, req, realm)
+                return HTTPBasicAuthHandler.retry_http_basic_auth(self, host, req, realm)
 
         authhandler_class = OscHTTPBasicAuthHandler
 
@@ -470,15 +488,15 @@ def _build_opener(url):
     # with None as first argument, it will always use this username/password
     # combination for urls for which arg2 (apisrv) is a super-url
     authhandler = authhandler_class( \
-        urllib2.HTTPPasswordMgrWithDefaultRealm())
+        HTTPPasswordMgrWithDefaultRealm())
     authhandler.add_password(None, apiurl, options['user'], options['pass'])
 
     if options['sslcertck']:
         try:
-            import oscssl
+            from . import oscssl
             from M2Crypto import m2urllib2
-        except ImportError, e:
-            print e
+        except ImportError as e:
+            print(e)
             raise NoSecureSSLError('M2Crypto is needed to access %s in a secure way.\nPlease install python-m2crypto.' % apiurl)
 
         cafile = options.get('cafile', None)
@@ -491,13 +509,15 @@ def _build_opener(url):
                 elif os.path.isdir(i):
                     capath = i
                     break
+        if not cafile and not capath:
+            raise Exception('No CA certificates found')
         ctx = oscssl.mySSLContext()
         if ctx.load_verify_locations(capath=capath, cafile=cafile) != 1:
             raise Exception('No CA certificates found')
-        opener = m2urllib2.build_opener(ctx, oscssl.myHTTPSHandler(ssl_context=ctx, appname='osc'), urllib2.HTTPCookieProcessor(cookiejar), authhandler, proxyhandler)
+        opener = m2urllib2.build_opener(ctx, oscssl.myHTTPSHandler(ssl_context=ctx, appname='osc'), HTTPCookieProcessor(cookiejar), authhandler, proxyhandler)
     else:
-        print >>sys.stderr, "WARNING: SSL certificate checks disabled. Connection is insecure!\n"
-        opener = urllib2.build_opener(urllib2.HTTPCookieProcessor(cookiejar), authhandler, proxyhandler)
+        print("WARNING: SSL certificate checks disabled. Connection is insecure!\n", file=sys.stderr)
+        opener = build_opener(HTTPCookieProcessor(cookiejar), authhandler, proxyhandler)
     opener.addheaders = [('User-agent', 'osc/%s' % __version__)]
     _build_opener.last_opener = (apiurl, opener)
     return opener
@@ -509,12 +529,13 @@ def init_basicauth(config):
     def filterhdrs(meth, ishdr, *hdrs):
         # this is so ugly but httplib doesn't use
         # a logger object or such
-        def new_method(*args, **kwargs):
-            stdout = sys.stdout
-            sys.stdout = StringIO.StringIO()
-            meth(*args, **kwargs)
+        def new_method(self, *args, **kwargs):
+            self._orig_stdout = sys.stdout
+            sys.stdout = StringIO()
+            meth(self, *args, **kwargs)
             hdr = sys.stdout.getvalue()
-            sys.stdout = stdout
+            sys.stdout = self._orig_stdout
+            del self._orig_stdout
             for i in hdrs:
                 if ishdr:
                     hdr = re.sub(r'%s:[^\\r]*\\r\\n' % i, '', hdr)
@@ -525,8 +546,8 @@ def init_basicauth(config):
         return new_method
 
     if config['http_debug'] and not config['http_full_debug']:
-        httplib.HTTPConnection.send = filterhdrs(httplib.HTTPConnection.send, True, 'Cookie', 'Authorization')
-        httplib.HTTPResponse.begin = filterhdrs(httplib.HTTPResponse.begin, False, 'header: Set-Cookie.*\n')
+        HTTPConnection.send = filterhdrs(HTTPConnection.send, True, 'Cookie', 'Authorization')
+        HTTPResponse.begin = filterhdrs(HTTPResponse.begin, False, 'header: Set-Cookie.*\n')
 
     if sys.version_info < (2, 6):
         # HTTPS proxy is not supported in old urllib2. It only leads to an error
@@ -540,20 +561,20 @@ def init_basicauth(config):
         # brute force
         def urllib2_debug_init(self, debuglevel=0):
             self._debuglevel = 1
-        urllib2.AbstractHTTPHandler.__init__ = urllib2_debug_init
+        AbstractHTTPHandler.__init__ = urllib2_debug_init
 
     cookie_file = os.path.expanduser(config['cookiejar'])
     global cookiejar
-    cookiejar = cookielib.LWPCookieJar(cookie_file)
+    cookiejar = LWPCookieJar(cookie_file)
     try:
         cookiejar.load(ignore_discard=True)
     except IOError:
         try:
             open(cookie_file, 'w').close()
-            os.chmod(cookie_file, 0600)
+            os.chmod(cookie_file, 0o600)
         except:
             #print 'Unable to create cookiejar file: \'%s\'. Using RAM-based cookies.' % cookie_file
-            cookiejar = cookielib.CookieJar()
+            cookiejar = CookieJar()
 
 
 def get_configParser(conffile=None, force_read=False):
@@ -582,7 +603,7 @@ def write_config(fname, cp):
         cp.write(f, comments=True)
     try:
         os.rename(fname + '.new', fname)
-        os.chmod(fname, 0600)
+        os.chmod(fname, 0o600)
     except:
         if os.path.exists(fname + '.new'):
             os.unlink(fname + '.new')
@@ -675,7 +696,7 @@ def write_initial_config(conffile, entries, custom_template=''):
     else:
         config['passx'] = base64.b64encode(config['pass'].encode('bz2'))
 
-    sio = StringIO.StringIO(conf_template.strip() % config)
+    sio = StringIO(conf_template.strip() % config)
     cp = OscConfigParser.OscConfigParser(DEFAULTS)
     cp.readfp(sio)
     write_config(conffile, cp)
@@ -743,7 +764,7 @@ def get_config(override_conffile=None,
     # okay, we made sure that .oscrc exists
 
     # make sure it is not world readable, it may contain a password.
-    os.chmod(conffile, 0600)
+    os.chmod(conffile, 0o600)
 
     cp = get_configParser(conffile)
 
@@ -759,7 +780,7 @@ def get_config(override_conffile=None,
     for i in boolean_opts:
         try:
             config[i] = cp.getboolean('general', i)
-        except ValueError, e:
+        except ValueError as e:
             raise oscerr.ConfigError('cannot parse \'%s\' setting: ' % i + str(e), conffile)
 
     config['packagecachedir'] = os.path.expanduser(config['packagecachedir'])
@@ -791,6 +812,7 @@ def get_config(override_conffile=None,
         scheme, host = parse_apisrv_url(config.get('scheme', 'https'), url)
         apiurl = urljoin(scheme, host)
         user = None
+        password = None
         if config['use_keyring'] and GENERIC_KEYRING:
             try:
                 # Read from keyring lib if available
@@ -810,15 +832,22 @@ def get_config(override_conffile=None,
                     password = gk_data[0]['password']
                 else:
                     # this is most likely an error
-                    print >>sys.stderr, 'warning: no password found in keyring'
+                    print('warning: no password found in keyring', file=sys.stderr)
             except gnomekeyring.NoMatchError:
                 # Fallback to file based auth.
                 pass
 
         if not user is None and len(user) == 0:
             user = None
-            print >>sys.stderr, 'Warning: blank user in the keyring for the ' \
-                'apiurl %s.\nPlease fix your keyring entry.'
+            print('Warning: blank user in the keyring for the ' \
+                'apiurl %s.\nPlease fix your keyring entry.', file=sys.stderr)
+
+        if user is not None and password is None:
+            err = ('no password defined for "%s".\nPlease fix your keyring '
+                   'entry or gnome-keyring setup.\nAssuming an empty password.'
+                   % url)
+            print(err, file=sys.stderr)
+            password = ''
 
         # Read credentials from config
         if user is None:
@@ -840,12 +869,12 @@ def get_config(override_conffile=None,
             if config['plaintext_passwd'] and passwordx or not config['plaintext_passwd'] and password:
                 if config['plaintext_passwd']:
                     if password != passwordx:
-                        print >>sys.stderr, '%s: rewriting from encoded pass to plain pass' % url
+                        print('%s: rewriting from encoded pass to plain pass' % url, file=sys.stderr)
                     add_section(conffile, url, user, passwordx)
                     password = passwordx
                 else:
                     if password != passwordx:
-                        print >>sys.stderr, '%s: rewriting from plain pass to encoded pass' % url
+                        print('%s: rewriting from plain pass to encoded pass' % url, file=sys.stderr)
                     add_section(conffile, url, user, password)
 
             if not config['plaintext_passwd']:
@@ -902,10 +931,10 @@ def get_config(override_conffile=None,
         scheme = config.get('scheme', 'https')
         config['apiurl'] = urljoin(scheme, apisrv)
     if 'apisrc' in config or 'scheme' in config:
-        print >>sys.stderr, 'Warning: Use of the \'scheme\' or \'apisrv\' in ~/.oscrc is deprecated!\n' \
-                            'Warning: See README for migration details.'
+        print('Warning: Use of the \'scheme\' or \'apisrv\' in ~/.oscrc is deprecated!\n' \
+                            'Warning: See README for migration details.', file=sys.stderr)
     if 'build_platform' in config:
-        print >>sys.stderr, 'Warning: Use of \'build_platform\' config option is deprecated! (use \'build_repository\' instead)'
+        print('Warning: Use of \'build_platform\' config option is deprecated! (use \'build_repository\' instead)', file=sys.stderr)
         config['build_repository'] = config['build_platform']
 
     config['verbose'] = int(config['verbose'])
@@ -934,7 +963,7 @@ def get_config(override_conffile=None,
     # provided that there _are_ credentials for the chosen apiurl:
     try:
         config['user'] = get_apiurl_usr(config['apiurl'])
-    except oscerr.ConfigMissingApiurl, e:
+    except oscerr.ConfigMissingApiurl as e:
         e.msg = config_missing_apiurl_text % config['apiurl']
         e.file = conffile
         raise e
