@@ -21,6 +21,7 @@ import test_package_status
 import test_project_status
 import test_request
 import test_setlinkrev
+import test_prdiff
 
 suite = unittest.TestSuite()
 suite.addTests(test_addfiles.suite())
@@ -36,6 +37,7 @@ suite.addTests(test_package_status.suite())
 suite.addTests(test_project_status.suite())
 suite.addTests(test_request.suite())
 suite.addTests(test_setlinkrev.suite())
+suite.addTests(test_prdiff.suite())
 
 if have_xmlrunner:
     result = xmlrunner.XMLTestRunner(output=os.path.join(os.getcwd(), 'junit-xml-results')).run(suite)
